package java.util.stream;

import java.util.Objects;
import java.util.function.DoubleConsumer;
import java.util.function.LongConsumer;
import java.util.function.IntConsumer;
import java.util.function.Consumer;

interface Sink<T> extends Consumer<T>
{
    default public void begin(long n) {
    }
    
    default public void end() {
    }
    
    default public boolean cancellationRequested() {
        return false;
    }
    
    default public void accept(int n) {
        throw new IllegalStateException("called wrong accept method");
    }
    
    default public void accept(long n) {
        throw new IllegalStateException("called wrong accept method");
    }
    
    default public void accept(double n) {
        throw new IllegalStateException("called wrong accept method");
    }
    
    public interface OfInt extends Sink<Integer>, IntConsumer
    {
        public abstract void accept(int p0);
        
        default public void accept(Integer n) {
            if (Tripwire.ENABLED) {
                Tripwire.trip(this.getClass(), "{0} calling Sink.OfInt.accept(Integer)");
            }
            this.accept((int)n);
        }
    }
    
    public interface OfLong extends Sink<Long>, LongConsumer
    {
        public abstract void accept(long p0);
        
        default public void accept(Long n) {
            if (Tripwire.ENABLED) {
                Tripwire.trip(this.getClass(), "{0} calling Sink.OfLong.accept(Long)");
            }
            this.accept((long)n);
        }
    }
    
    public interface OfDouble extends Sink<Double>, DoubleConsumer
    {
        public abstract void accept(double p0);
        
        default public void accept(Double n) {
            if (Tripwire.ENABLED) {
                Tripwire.trip(this.getClass(), "{0} calling Sink.OfDouble.accept(Double)");
            }
            this.accept((double)n);
        }
    }
    
    public abstract static class ChainedReference<T, E_OUT> implements Sink<T>
    {
        protected final Sink<? super E_OUT> downstream;
        
        public ChainedReference(Sink<? super E_OUT> obj) {
            this.downstream = (Sink<? super E_OUT>)Objects.requireNonNull(obj);
        }
        
        @Override
        public void begin(long n) {
            this.downstream.begin(n);
        }
        
        @Override
        public void end() {
            this.downstream.end();
        }
        
        @Override
        public boolean cancellationRequested() {
            return this.downstream.cancellationRequested();
        }
    }
    
    public abstract static class ChainedInt<E_OUT> implements Sink.OfInt
    {
        protected final Sink<? super E_OUT> downstream;
        
        public ChainedInt(Sink<? super E_OUT> obj) {
            this.downstream = (Sink<? super E_OUT>)Objects.requireNonNull(obj);
        }
        
        @Override
        public void begin(long n) {
            this.downstream.begin(n);
        }
        
        @Override
        public void end() {
            this.downstream.end();
        }
        
        @Override
        public boolean cancellationRequested() {
            return this.downstream.cancellationRequested();
        }
    }
    
    public abstract static class ChainedLong<E_OUT> implements Sink.OfLong
    {
        protected final Sink<? super E_OUT> downstream;
        
        public ChainedLong(Sink<? super E_OUT> obj) {
            this.downstream = (Sink<? super E_OUT>)Objects.requireNonNull(obj);
        }
        
        @Override
        public void begin(long n) {
            this.downstream.begin(n);
        }
        
        @Override
        public void end() {
            this.downstream.end();
        }
        
        @Override
        public boolean cancellationRequested() {
            return this.downstream.cancellationRequested();
        }
    }
    
    public abstract static class ChainedDouble<E_OUT> implements Sink.OfDouble
    {
        protected final Sink<? super E_OUT> downstream;
        
        public ChainedDouble(Sink<? super E_OUT> obj) {
            this.downstream = (Sink<? super E_OUT>)Objects.requireNonNull(obj);
        }
        
        @Override
        public void begin(long n) {
            this.downstream.begin(n);
        }
        
        @Override
        public void end() {
            this.downstream.end();
        }
        
        @Override
        public boolean cancellationRequested() {
            return this.downstream.cancellationRequested();
        }
    }
}
